/* Torneo de Organismos
   Inspirado en el programa Robocode de IBM, esta simulacin consiste en generar N tanques
   en un tablero (un plano), cada tanque puede girar, desplazarse hacia adelante o hacia atrs,
   tiene un radar y disparar. Mientras dura el torneo, en cada ciclo, el tanque va perdiendo
   un punto de vida, pierde tres puntos de vida si es impactado por un proyectil de otro tanque
   y gana tres puntos de vida si al disparar acierta a otro tanque.

   El tanque tiene cuatro(4) eventos que se activan cuando:
   1. Es alcanzado por un disparo.
   2. Acierta su disparo en otro tanque.
   3. Choca contra otro tanque.
   4. Detecta otro tanque con el radar.

   Cada evento dispara una serie de instrucciones que han sido previamente generadas al azar.

   En esas instrucciones, tpicas de un lenguaje muy simple de programacin se manejan:
   1. Diez(10) variables.
   2. Las instrucciones IF y SET (asignacin)
   3. Nueve(9) funciones:
			a: Avanzar_Tanque();
			b: Retroceder_Tanque();
			c: Girar_Tanque(1-8);
			d: Girar_Radar(1-8);
			e: Disparar();  disminuye vida en un punto
			f: GetPosX();
			g: GetPosY();
			h: GetVida();
			i: Reproducirse();
 

  Observaciones:  Este programa se crea como base de desarrollos futuros para probar conceptos
    sobre la evolucin: macroorganismos, reproduccin, sensores, emociones, etc..
    Fue diseado para ser lo ms rpido posible.
    

  Versin: 4.0
  Fecha: 05 de Diciembre de 2004
  
  Autor: Rafael Alberto Moreno Parra
  E-mail: enginelife@hotmail.com
  WebPage: http://www.geocities.com/krousky


  En esta nueva simulacin hay estos cambios
	Un ciclo generar tanques al azar y solo sobrevivir uno.
	El que sobrevive engrosar el podium de los mejores.

    Cuando el podium de los mejores este lleno, entonces
	se mostrar en estadstica que paquetes de instrucciones fueron mas usados
	y en que eventos.
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "Tanque.h"
#include "Instruccion.h"
#include "Inicializa.h"
#include "MejorTanque.h"

#define MAXTANQUES 4000 // 49bytes  * 4000 = 196000 => 200Kb
#define MAXINSTR 200 //    522bytes *  200 = 104400 => 110Kb
#define NUMEVENTOS 5


Inicializa objIni;
Tanque objTanque[MAXTANQUES];
MejorTanque objMejor[MAXTANQUES];
Instruccion objInst[MAXINSTR];
int objRelacion[MAXTANQUES][NUMEVENTOS];


void vSimulacion();
void vGeneraTanques(void);
void vCreaTanque(unsigned int);
void vGeneraInstrucciones(void);
void vCreaInstruccion(unsigned int);
bool bChequeaColision(unsigned int);

bool bImpactado(unsigned int);
bool bAciertaDisparo(unsigned int);
bool bChequeaColision(unsigned int);
bool bDetectaRadar(unsigned int);

void vEjecutaInstrucciones(unsigned int, unsigned int);
void vAvance(unsigned int, signed int *, signed int *);
void vInforme(unsigned int);
void vMuestraInstrucciones(unsigned int);


void main()
{
	unsigned int iTanque; /* Para ir de tanque en tanque */
	unsigned int iEvento; /* Para crear la relacin tanque-evento-bloque */
	unsigned int iCopiaRel; /* Copia la relacin tanque-evento-bloque en el podium de los mejores */
	unsigned int iInstr; /* Sirve para contabilizar que bloque de instrucciones on usadas por los tanques exitosos */

	//Guarda la probabilidad de ser escogido
	unsigned int iProbabilidad[MAXINSTR];
	for (unsigned int iCont=0; iCont<MAXINSTR; iCont++)
		iProbabilidad[iCont] = 1;

	/* Lee el archivo plano de parmetros de simulaciones */
	objIni.vPantallaIni();
	objIni.vLeeArchivoIni();
	objIni.vInforme();

	/* Inicializa la semilla de numeros aleatorios */
	time_t ltime2;
	time(&ltime2);
	srand( ltime2 );
	
	/* Crea las instrucciones. Permanecen inalterables. */
	vGeneraInstrucciones();

	/* Ejecuta la simulacin con los tanques aleatorios */
	for (unsigned int iSimula=0; iSimula < objIni.stDatVA.iTorneos; iSimula++)
	{
		/* Coloca los tanques en posiciones, radar, direccin al azar con determinada energa */
		vGeneraTanques();

		/* Crea la relacin Tanque + Evento + Instruccion
		   El cambio es que los organismos simples mas exitosos, tienen mas
		   posibilidades de ser escogidos para formar nuevos MacroOrganismos */
        
		/* Funciona as:
			Organismo Simple		Veces Usado
			0						3
			1						1
			2						8
			3						10
			4						5

			Sumo las Veces Usado: 3+1+8+10+5 = 27

			Genero un nmero al azar entre 1 y 27, por ejemplo 13
            Luego empiezo a ver donde caera ese 13..
			Organismo simple 0 : 3  (3>=13? False)
			Organismo simple 1 : 3+1 (4>=13? False)
			Organismo simple 2 : 3+1+8 (12>=13? False)
			Organismo simple 3 : 3+1+8+10 (22>=13? True)

			Entonces el organismo simple seleccionado es el 3
		*/
		unsigned int iAcumula =0;
		for (unsigned int iCont=0; iCont < objIni.stDatVA.iBloqueInstr; iCont++)
			iAcumula += iProbabilidad[iCont];
	
		for(iTanque=0; iTanque < objIni.stDatVA.iNumTanques; iTanque++)
			for (iEvento=0; iEvento < NUMEVENTOS; iEvento++)
			{
				unsigned int iNumeroAzar = rand()%iAcumula+1;
				unsigned int iAcum = 0;
				unsigned int iEscoge;
				for (iEscoge=0; iEscoge < objIni.stDatVA.iBloqueInstr; iEscoge++)
				{
					iAcum += iProbabilidad[iEscoge];
					if (iAcum >= iNumeroAzar) break;
				}
				objRelacion[iTanque][iEvento] = iEscoge;
			}

		/* Ejecuta la simulacin (un torneo). Solo sobrevive un tanque. */
		vSimulacion();
		
		/* Guarda el mejor tanque en el Podium */
		for (iTanque=0; iTanque < objIni.stDatVA.iNumTanques; iTanque++)
			if (objTanque[iTanque].iVida > 0) break;

		objMejor[iSimula].iCodigo = iTanque;
		for (iCopiaRel = 0; iCopiaRel < NUMEVENTOS; iCopiaRel++)
		{
			iInstr = objMejor[iSimula].iEvento[iCopiaRel] = objRelacion[iTanque][iCopiaRel];
			objInst[iInstr].iParticipa[iCopiaRel]++; //El bloque aumenta en calificacin por ser usado */
			iProbabilidad[iInstr]++; //El bloque aumenta su probabilidad de ser escogido en el proximo MacroOrganismo */
		}
		
		printf("Torneo: %d\n", iSimula);

		//Cada cierto tiempo se debe evaluar que organismos simples les ha ido muy bien
		//porque son reproducidos y que organismos simples no han sido escogidos
		//por lo cual deben ser eliminados por los ventajosos.
		//Ser en una prxima simulacin.
		if (iSimula%30==0)
		{
			time(&ltime2);
			srand( ltime2 );
		}
	}

	printf("\nEn cada torneo solo sobrevive un MacroOrganismo (tanque).\n");
	printf("Los organismos simples que conforman este tanque son premiados con un punto.\n");
	printf("El informe a continuacion muestra cuantas veces fue usado el\n");
	printf("simple organismo por los diferentes MacroOrganismos exitosos\n\n");

	for (unsigned int iInforme=0; iInforme < objIni.stDatVA.iBloqueInstr; iInforme++)
	{
		unsigned int iTotal = objInst[iInforme].iParticipa[0] + objInst[iInforme].iParticipa[1] + objInst[iInforme].iParticipa[2] + objInst[iInforme].iParticipa[3] + objInst[iInforme].iParticipa[4];
		printf("Organismo simple: %d, veces usado: %d\n", iInforme, iTotal);
		printf("   Se uso en el evento: Lo impactaron?: %d\n", objInst[iInforme].iParticipa[0]);
		printf("   Se uso en el evento: La bala impacto tanque enemigo?: %d\n", objInst[iInforme].iParticipa[1]);
		printf("   Se uso en el evento: Choco con otro tanque?: %d\n", objInst[iInforme].iParticipa[2]);
		printf("   Se uso en el evento: Detecto otro tanque con el radar?: %d\n", objInst[iInforme].iParticipa[3]);
		printf("   Se uso en ejecucion por defecto del tanque: %d\n\n", objInst[iInforme].iParticipa[4]);
	}
}		

void vSimulacion()
{	
	signed int iBalaX, iBalaY; /* Para mover la bala disparada por el tanque */
	unsigned int iTanque; /* Para ir de tanque en tanque */
	unsigned int iTanquesVivos; /* Que tanques estan vivitos y coleando */

	/* Ciclo de vida de los Tanques */
	iTanque = 0;
	while (true) /* Si solo queda un tanque vivo termina la simulacin */
	{
		/* Chequea cuantos tanques vivos quedaron */
		iTanquesVivos = 0;
		for (register unsigned int iCont=0; iCont < objIni.stDatVA.iNumTanques; iCont++)
			if (objTanque[iCont].iVida > 0) iTanquesVivos++;
		if (iTanquesVivos <= 1) break;

		/* Evalua cada tanque, cuando llegue al ultimo vuelve y arranca de cero */
		iTanque++;
		if (iTanque >= objIni.stDatVA.iNumTanques) iTanque = 0;

		/* Solo evalua al tanque vivo */
		if (objTanque[iTanque].iVida <= 0) continue;


		/* Lo impactaron? quita un punto de vida */
		if (bImpactado(iTanque))
		{
			objTanque[iTanque].iVida-=3;
			objTanque[iTanque].iAlcanzado++;

			//Ejecuta las instrucciones
			vEjecutaInstrucciones(iTanque, 0);
		}

		/* Instrucciones que hace el tanque sin necesidad de emociones */
		{		
			if (objTanque[iTanque].bEstadoBala)
			{
				vAvance(objTanque[iTanque].iDirBala, &iBalaX, &iBalaY);
				objTanque[iTanque].iPosBalaX += iBalaX;
				objTanque[iTanque].iPosBalaY += iBalaY;
			}	

			if (objTanque[iTanque].iPosBalaX < 0 || objTanque[iTanque].iPosBalaX >= objIni.stDatVA.iTableroX) objTanque[iTanque].bEstadoBala = false;
			if (objTanque[iTanque].iPosBalaY < 0 || objTanque[iTanque].iPosBalaY >= objIni.stDatVA.iTableroY) objTanque[iTanque].bEstadoBala = false;

			//Ejecuta instrucciones del Tanque
			vEjecutaInstrucciones(iTanque, 4);

			//Disminuye vida
			objTanque[iTanque].iVida--;
		}

		
		/* La bala disparada impact a un tanque enemigo? da tres puntos de vida */
		if (bAciertaDisparo(iTanque))
		{
			objTanque[iTanque].iVida+=3;
			objTanque[iTanque].iAcerto++;
			
			//Ejecuta las instrucciones
			vEjecutaInstrucciones(iTanque, 1);
		}

		/* Choc con otro tanque? */
		if (bChequeaColision(iTanque))
		{
			//Ejecuta las instrucciones
			vEjecutaInstrucciones(iTanque, 2);
		}

		/* Detect otro tanque con el radar? */
		if (bDetectaRadar(iTanque))
		{
			//Ejecuta las instrucciones
			vEjecutaInstrucciones(iTanque, 3);
		}

	}
}


/* Genera todos los tanques */
void vGeneraTanques()
{
	for (unsigned int iCont = 0; iCont < objIni.stDatVA.iNumTanques; iCont++)
		vCreaTanque(iCont);
}

/* Genera un tanque en particular */
void vCreaTanque(unsigned int iTanque)
{
	objTanque[iTanque].iCodigo = iTanque;

	do
	{
		objTanque[iTanque].iPosX = rand() % objIni.stDatVA.iTableroX;
		objTanque[iTanque].iPosY = rand() % objIni.stDatVA.iTableroY;
	} while (bChequeaColision(iTanque));

	objTanque[iTanque].iDireccion = rand() % 8;
	objTanque[iTanque].iRadar = rand() % 8;
	objTanque[iTanque].bEstadoBala = false;
	objTanque[iTanque].iVida = objIni.stDatVA.iConstVida;

	objTanque[iTanque].iAcerto = 0;
	objTanque[iTanque].iAlcanzado = 0;
	objTanque[iTanque].iReproduce = 0;
}

/* Genera todos los juegos de instrucciones */
void vGeneraInstrucciones()
{
	for (unsigned int iCont = 0; iCont < objIni.stDatVA.iBloqueInstr; iCont++)
		vCreaInstruccion(iCont);
}

/* Genera un juego de instrucciones en particular */
void vCreaInstruccion(unsigned int iInstr)
{
	objInst[iInstr].iParticipa[0] = 0;
	objInst[iInstr].iParticipa[1] = 0;
	objInst[iInstr].iParticipa[2] = 0;
	objInst[iInstr].iParticipa[3] = 0;
	objInst[iInstr].iParticipa[4] = 0;

	for (unsigned int iCont=0; iCont < TRIPLETES; iCont++)
	{
		/* 
			0:  IF >
			1:  IF <
			2:  IF =
			3:  IF <>
			4:  SET
			5:  Avanzar_Tanque();
			6:  Retroceder_Tanque();
			7:  Girar_Tanque(1-8);
			8:  Girar_Radar(1-8);
			9:  Disparar();  disminuye vida en un punto
			10: GetPosX();
			11: GetPosY();
			12: GetVida();
			13: Reproducirse();
        */
		objInst[iInstr].m_oTriplete[iCont].iInstruccion = abs(rand()%14);
		
		/* Variable que recibe el resultado de la funcin */
		objInst[iInstr].m_oTriplete[iCont].iRecibe = abs(rand()%VARIABLES);

		/* Chequea si el primer operando es variable o un nmero */
		if (abs(rand()%2) == 0 )
			objInst[iInstr].m_oTriplete[iCont].bEsNumero1 = true;
		else
			objInst[iInstr].m_oTriplete[iCont].bEsNumero1 = false;

		/* Chequea si el segundo operando es variable o un nmero */
		if (abs(rand()%2) == 0 )
			objInst[iInstr].m_oTriplete[iCont].bEsNumero2 = true;
		else
			objInst[iInstr].m_oTriplete[iCont].bEsNumero2 = false;

		/* Variable o nmero... decide cual, si es un nmero va de 0 a 9 */
		objInst[iInstr].m_oTriplete[iCont].iVariable1 = abs(rand()%VARIABLES);
		objInst[iInstr].m_oTriplete[iCont].iVariable2 = abs(rand()%VARIABLES);

		/* Decide operador */
		switch (abs(rand()%4))
		{
		case 0: objInst[iInstr].m_oTriplete[iCont].cOperador = '+'; break;
		case 1: objInst[iInstr].m_oTriplete[iCont].cOperador = '-'; break;
		case 2: objInst[iInstr].m_oTriplete[iCont].cOperador = '*'; break;
		case 3: objInst[iInstr].m_oTriplete[iCont].cOperador = '/'; break;
		}
		
		/* Salta a si hay una condicion IF */
		objInst[iInstr].m_oTriplete[iCont].iGotoLabel = abs(rand()%TRIPLETES);
	}
}


/* ************************************************************************************ */
/*                            SECCION DE EVENTOS O EMOCIONES                            */ 
/* ************************************************************************************ */

/* Chequea si al tanque lo impactaron */
bool bImpactado (unsigned int iTanque)
{
	for (unsigned int iCont = 0; iCont < objIni.stDatVA.iNumTanques; iCont++)
	{
		if (iCont != iTanque && objTanque[iCont].iVida > 0 && objTanque[iCont].bEstadoBala == true )
			if ( objTanque[iTanque].iPosX == objTanque[iCont].iPosBalaX &&
				 objTanque[iTanque].iPosY == objTanque[iCont].iPosBalaY )
			{
				objTanque[iCont].bEstadoBala = false; /* Puede disparar de nuevo el tanque que le acert primero */
				return true;
			}
	}

	return false;
}

/* Chequea si la bala disparada acert en un tanque enemigo */
bool bAciertaDisparo (unsigned int iTanque)
{
	if (objTanque[iTanque].bEstadoBala == false) return false;

	for (unsigned int iCont = 0; iCont < objIni.stDatVA.iNumTanques; iCont++)
	{
		if (iCont != iTanque && objTanque[iCont].iVida > 0)
			if ( objTanque[iTanque].iPosBalaX == objTanque[iCont].iPosX &&
				 objTanque[iTanque].iPosBalaY == objTanque[iCont].iPosY )
					return true;
	}

	return false;
}

/* Chequea si el tanque esta colisionando con otro tanque vivo */
bool bChequeaColision (unsigned int iTanque)
{
	for (unsigned int iCont = 0; iCont < objIni.stDatVA.iNumTanques; iCont++)
	{
		if (iCont != iTanque && objTanque[iCont].iVida > 0 )
			if ( objTanque[iTanque].iPosX == objTanque[iCont].iPosX &&
				 objTanque[iTanque].iPosY == objTanque[iCont].iPosY )
					return true;
	}

	return false;
}
		
/* Detect otro tanque con el radar? */
bool bDetectaRadar (unsigned int iTanque)
{
	signed int iRastreoX, iRastreoY;
	unsigned int iPosRadarX, iPosRadarY;

	/* Depende de la posicin que tiene el radar */
	vAvance(objTanque[iTanque].iRadar, &iRastreoX, &iRastreoY);

	/* Desde la posicin del tanque hasta los lmites del tablero */
	iPosRadarX = objTanque[iTanque].iPosX;
	iPosRadarY = objTanque[iTanque].iPosY;

	while (iPosRadarX >= 0 && iPosRadarX < objIni.stDatVA.iTableroX && iPosRadarY >= 0 && iPosRadarY < objIni.stDatVA.iTableroY)
	{
		iPosRadarX += iRastreoX;
		iPosRadarY += iRastreoY;
		
		for (unsigned int iCont = 0; iCont < objIni.stDatVA.iNumTanques; iCont++)
		{
			if (iCont != iTanque && objTanque[iCont].iVida > 0 )
				if ( objTanque[iCont].iPosX == iPosRadarX &&
					 objTanque[iCont].iPosY == iPosRadarY )
						return true;
		}
	}
    return false;
}

/* ***************************************************************************************** */
/* EJECUTAR INSTRUCCIONES DEL TANQUE 
/* ***************************************************************************************** */
void vEjecutaInstrucciones(unsigned int iTanque, unsigned int iEvento)
{
	unsigned int iInstr, iCont=0, iVarRecibe, iCiclo=0;
	signed int iIncX, iIncY, iValor1, iValor2, iValorT;
	bool bCiclo = true;

	iInstr = objRelacion[iTanque][iEvento]; //Trae las instrucciones para este tanque y evento (cruce)

	while(iCont<TRIPLETES && bCiclo)
	{
		/* Evita que se quede en un ciclo infinito */
		iCiclo++;
		if (iCiclo > objIni.stDatVA.iMaxCiclos) bCiclo = false;

		/* Variable que recibe el resultado de la funcin */
		iVarRecibe = objInst[iInstr].m_oTriplete[iCont].iRecibe;

		/* Las intrucciones IF se interpretan as:
			IF varRecibe > var1  + var2 THEN Goto Etiqueta 
			IF varRecibe > 6557  - var2 THEN Goto Etiqueta
	    */
	    if (objInst[iInstr].m_oTriplete[iCont].bEsNumero1)
			iValor1 = objInst[iInstr].m_oTriplete[iCont].iVariable1;
		else
			iValor1 = objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable1];
			
		if (objInst[iInstr].m_oTriplete[iCont].bEsNumero2)
			iValor2 = objInst[iInstr].m_oTriplete[iCont].iVariable2;
		else
			iValor2 = objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable2];

		switch (objInst[iInstr].m_oTriplete[iCont].cOperador)
		{
			 case '+': iValorT = iValor1 + iValor2; break;
			 case '-': iValorT = iValor1 - iValor2; break;
			 case '*': iValorT = iValor1 * iValor2; break;
			 case '/': if (iValor2!=0)
						   iValorT = iValor1 / iValor2;
					   else
						   iValorT = 0;
					   break;
		}

		switch (objInst[iInstr].m_oTriplete[iCont].iInstruccion)
		{
		case 0:  //printf("IF > \n");
				 if (objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable1] > iValorT)
					 iCont = objInst[iInstr].m_oTriplete[iCont].iGotoLabel;
				 break; /* IF > */

		case 1:  //printf("IF < \n");
				 if (objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable1] < iValorT)
					 iCont = objInst[iInstr].m_oTriplete[iCont].iGotoLabel;
				 break; /* IF < */

		case 2:  //printf("IF ==  \n");
				 if (objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable1] == iValorT)
					 iCont = objInst[iInstr].m_oTriplete[iCont].iGotoLabel;
				 break; /* IF == */

		case 3:  //printf("IF <>  \n");
				 if (objInst[iInstr].iVariable[objInst[iInstr].m_oTriplete[iCont].iVariable1] != iValorT)
					 iCont = objInst[iInstr].m_oTriplete[iCont].iGotoLabel;
				 break; /* IF <> */

		case 4:  //printf("SET  \n");
				 objInst[iInstr].iVariable[iVarRecibe] = iValorT;
				 break; /* SET */

		case 5:  //printf("Avanzar Tanque \n");
				 vAvance(objTanque[iTanque].iDireccion, &iIncX, &iIncY);
				 objTanque[iTanque].iPosX += iIncX;
				 objTanque[iTanque].iPosY += iIncY;
				 if (objTanque[iTanque].iPosX < 0) objTanque[iTanque].iPosX = 0;
				 if (objTanque[iTanque].iPosX >= objIni.stDatVA.iTableroX) objTanque[iTanque].iPosX = objIni.stDatVA.iTableroX - 1;
				 if (objTanque[iTanque].iPosY < 0) objTanque[iTanque].iPosY = 0;
				 if (objTanque[iTanque].iPosY >= objIni.stDatVA.iTableroY) objTanque[iTanque].iPosY = objIni.stDatVA.iTableroY - 1;
				 break; /* Avanzar_Tanque */

		case 6:  //printf("Retroceder Tanque \n");
				 vAvance(objTanque[iTanque].iDireccion, &iIncX, &iIncY);
				 objTanque[iTanque].iPosX -= iIncX;
				 objTanque[iTanque].iPosY -= iIncY;
				 if (objTanque[iTanque].iPosX < 0) objTanque[iTanque].iPosX = 0;
				 if (objTanque[iTanque].iPosX >= objIni.stDatVA.iTableroX) objTanque[iTanque].iPosX = objIni.stDatVA.iTableroX - 1;
				 if (objTanque[iTanque].iPosY < 0) objTanque[iTanque].iPosY = 0;
				 if (objTanque[iTanque].iPosY >= objIni.stDatVA.iTableroY) objTanque[iTanque].iPosY = objIni.stDatVA.iTableroY - 1;
				 break; /* Retroceder Tanque */

		case 7:  //printf("Girar Tanque \n");
			     objTanque[iTanque].iDireccion = objInst[iInstr].iVariable[iVarRecibe];
				 break; /* Girar_Tanque(variable_recibe) */

		case 8:  //printf("Girar Radar \n");
				 objTanque[iTanque].iRadar = objInst[iInstr].iVariable[iVarRecibe];
				 break; /* Girar_Radar(variable_recibe) */

		case 9:	 //printf("Disparar \n");
			     if (objTanque[iTanque].bEstadoBala == false)
				 {
					objTanque[iTanque].bEstadoBala = true;
					objTanque[iTanque].iPosBalaX = objTanque[iTanque].iPosX;
					objTanque[iTanque].iPosBalaY = objTanque[iTanque].iPosY;
					objTanque[iTanque].iDirBala  = objTanque[iTanque].iDireccion;
					objTanque[iTanque].iVida--; /* Penaliza con un punto el disparar */
				 } 
				 break; /* Disparar */

		case 10: //printf("GetPosX \n");
			     objInst[iInstr].iVariable[iVarRecibe] = objTanque[iTanque].iPosX;
			     break; /* GetPosX */

		case 11: //printf("GetPosY \n");
			     objInst[iInstr].iVariable[iVarRecibe] = objTanque[iTanque].iPosY;
			     break; /* GetPosY */

		case 12: //printf("GetVida \n");
				 objInst[iInstr].iVariable[iVarRecibe] = objTanque[iTanque].iVida;
			     break; /* GetVida */

		case 13: // printf("Reproducirse \n");
				if (objTanque[iTanque].iVida >= objIni.stDatVA.iLimReproduce)
				{
					for (unsigned int iReprod = 0; iReprod < objIni.stDatVA.iNumTanques; iReprod++)
					{
						if (objTanque[iReprod].iVida <= 0) //sobreescribe un tanque muerto
						{
							objTanque[iTanque].iReproduce++;

							vCreaTanque(iReprod);
							objTanque[iReprod].iVida = objTanque[iTanque].iVida / 2;
							objTanque[iTanque].iVida /= 2;

							//Copia los eventos
							for ( unsigned int iEvento=0; iEvento < NUMEVENTOS; iEvento++)
								objRelacion[iReprod][iEvento] = objRelacion[iTanque][iEvento];


							if (objIni.stDatVA.bMutacion)
							{
								unsigned int iMuta = rand() % NUMEVENTOS;
								objRelacion[iReprod][iMuta] = rand()%objIni.stDatVA.iBloqueInstr;
							}

							break;
						}
					}
				}
				break; /* Reproducirse */
		}
		iCont++;
	}

}


/* Retorna que valores debe adicionar en X y Y para avanzar en la Matriz */
void vAvance(unsigned int iDireccion, signed int *iIncX, signed int *iIncY)
{
	switch(iDireccion)
	{
	case 0: *iIncX =  0; *iIncY = -1; break; /* Vertical hacia arriba */
	case 1: *iIncX =  1; *iIncY = -1; break; /* Diagonal derecha hacia arriba */
	case 2: *iIncX =  1; *iIncY =  0; break; /* Horizontal derecha */
	case 3: *iIncX =  1; *iIncY =  1; break; /* Diagonal derecha hacia abajo */
	case 4: *iIncX =  0; *iIncY =  1; break; /* Vertical hacia abajo */
	case 5: *iIncX = -1; *iIncY =  1; break; /* Diagonal izquierda hacia abajo */
	case 6: *iIncX = -1; *iIncY =  0; break; /* Horizontal izquierda */
	default: *iIncX = -1; *iIncY = -1; break; /* Diagonal izquierda hacia arriba */
	}
}


/* Informe sobre energa e instrucciones del Tanque */
void vInforme(unsigned int iTanque)
{
	printf("iTanque: %d, iPosX: %d, iPosY: %d, iVida: %d\n", iTanque, objTanque[iTanque].iPosX, objTanque[iTanque].iPosY, objTanque[iTanque].iVida);
	
	printf("\nFue Impactado entonces ejecuta bloque: [%d]\n", objRelacion[iTanque][0]);
	vMuestraInstrucciones(objRelacion[iTanque][0]);

	printf("\nLe acerto a otro tanque entonces ejecuta bloque: [%d]\n", objRelacion[iTanque][1]);
	vMuestraInstrucciones(objRelacion[iTanque][1]);

	printf("\nChoco con un tanque entonces ejecuta bloque: [%d]\n", objRelacion[iTanque][2]);
	vMuestraInstrucciones(objRelacion[iTanque][2]);

	printf("\nDetecto el Radar entonces ejecuta bloque: [%d]\n", objRelacion[iTanque][3]);
	vMuestraInstrucciones(objRelacion[iTanque][3]);

	printf("\nVida Normal entonces ejecuta bloque: [%d]\n", objRelacion[iTanque][4]);
	vMuestraInstrucciones(objRelacion[iTanque][4]);
}

/* Muestra las instrucciones formateadas (creadas al azar) */
void vMuestraInstrucciones(unsigned int iInstr)
{
	char sbADN[1000], sExprTemp1[250], sExprTemp2[250]="";
	char cEnter[4];
    cEnter[0]= '\n';
    cEnter[1]= '\0';

	strcpy(sbADN,"#include <stdio.h>");
	strcat(sbADN,cEnter);
	strcat(sbADN,"void main()");
	strcat(sbADN,cEnter);
	strcat(sbADN,"{");
	strcat(sbADN,cEnter);
	strcat(sbADN,"  int V0=0, V1=0, V2=0, V3=0, V4=0, V5=0, V6=0, V7=0, V8=0, V9=0;");
	strcat(sbADN,cEnter);

	for (unsigned int iCont=1; iCont < TRIPLETES; iCont++)
    {
		sprintf(sExprTemp1, "  L%d: ", iCont);
		strcat(sbADN, sExprTemp1);

		if (objInst[iInstr].m_oTriplete[iCont].iInstruccion >= 5 ) //Es una funcion
		{
			switch(objInst[iInstr].m_oTriplete[iCont].iInstruccion)
			{
			case 5: sprintf(sExprTemp1, "Avanzar_Tanque();" ); break;
			case 6: sprintf(sExprTemp1, "Retroceder_Tanque();" ); break;
			case 7: sprintf(sExprTemp1, "Girar_Tanque(V%d);", objInst[iInstr].m_oTriplete[iCont].iRecibe ); break;
			case 8: sprintf(sExprTemp1, "Girar_Radar(V%d);", objInst[iInstr].m_oTriplete[iCont].iRecibe ); break;
			case 9: sprintf(sExprTemp1, "Disparar();" ); break;
			case 10: sprintf(sExprTemp1, "V%d = GetPosX();", objInst[iInstr].m_oTriplete[iCont].iRecibe); break;
			case 11: sprintf(sExprTemp1, "V%d = GetPosY();", objInst[iInstr].m_oTriplete[iCont].iRecibe); break;
			case 12: sprintf(sExprTemp1, "V%d = GetVida();", objInst[iInstr].m_oTriplete[iCont].iRecibe); break;
			case 13: sprintf(sExprTemp1, "Reproducirse();" ); break;
			}
		
			strcat(sbADN, sExprTemp1);
			strcat(sbADN, cEnter);
			continue;
		}
                 
        //Organiza los if y asignaciones
		if (objInst[iInstr].m_oTriplete[iCont].iInstruccion >=0 && objInst[iInstr].m_oTriplete[iCont].iInstruccion <= 3)
			strcat(sbADN, "if (");

		sprintf(sExprTemp1, "V%d ", objInst[iInstr].m_oTriplete[iCont].iRecibe);
		strcat(sbADN, sExprTemp1);

		switch (objInst[iInstr].m_oTriplete[iCont].iInstruccion)
		{
			case 0:	strcat(sbADN, "> ("); break;
			case 1:	strcat(sbADN, "< ("); break;
			case 2:	strcat(sbADN, "== ("); break;
			case 3:	strcat(sbADN, "!= ("); break;
			case 4:	strcat(sbADN, "= ("); break;
		}

		if (objInst[iInstr].m_oTriplete[iCont].bEsNumero1 == false)
			sprintf(sExprTemp1, "V%d", objInst[iInstr].m_oTriplete[iCont].iVariable1);
		else
			sprintf(sExprTemp1,  "%d", objInst[iInstr].m_oTriplete[iCont].iVariable1);

		strcat(sbADN, sExprTemp1);
		sprintf(sExprTemp1, " %c ", objInst[iInstr].m_oTriplete[iCont].cOperador);
		strcat(sbADN, sExprTemp1);

		if (objInst[iInstr].m_oTriplete[iCont].bEsNumero2 == false)
			sprintf(sExprTemp1, "V%d)", objInst[iInstr].m_oTriplete[iCont].iVariable2);
		else
			sprintf(sExprTemp1,  "%d)", objInst[iInstr].m_oTriplete[iCont].iVariable2);
		strcat(sbADN, sExprTemp1);

		if (objInst[iInstr].m_oTriplete[iCont].iInstruccion>=0 && objInst[iInstr].m_oTriplete[iCont].iInstruccion <= 3)
		{
			sprintf(sExprTemp1, ") goto L%d;", objInst[iInstr].m_oTriplete[iCont].iGotoLabel);
			strcat(sbADN, sExprTemp1);
		}
		else
			strcat(sbADN, ";");

		strcat(sbADN, cEnter);
	}
	strcat(sbADN, "  L0: ");
	strcat(sbADN, cEnter);
	strcat(sbADN, "}");
	strcat(sbADN, cEnter);
	printf(sbADN);
}
